import mysql.connector
import datetime
import xlwt


class MySheet:
  def __init__(self, sheet):
    self.sheet = sheet
    self.rowCnt = 0
    self.colCnt = 0
    self.widthFactor = 268
    self.dateStyle = xlwt.XFStyle()
    self.dateStyle.num_format_str = 'DD.MM.YYYY hh:mm:ss'
    
  
  def write(self, txt, style=None):
    if isinstance(txt, datetime.datetime):
      self.sheet.row(self.rowCnt).write(self.colCnt, txt, self.dateStyle)
    else:
      self.sheet.row(self.rowCnt).write(self.colCnt, txt)
    l = len(unicode(txt)) * self.widthFactor
    if l > self.sheet.col(self.colCnt).width:
      self.sheet.col(self.colCnt).width = l 
  
  def writeFirstFirst(self, txt, style=None):
    self.rowCnt = 0
    self.colCnt = 0
    self.write(txt, style)
    
  def writeNextFirst(self, txt, style=None):
    self.rowCnt += 1
    self.colCnt = 0
    self.write(txt, style)
  
  def writeSameNext(self, txt, style=None):
    self.colCnt += 1
    self.write(txt, style)
  
  
dbh = None
cur = None
xls = None
sheet = None

try:
  dbh = mysql.connector.connect(user='statsuser', password='test123', host='192.168.3.22', database='statsdb')
  cur = dbh.cursor()
  
  query = """
SELECT d.description, d.address,
       cd.ts, 
       cd.value, cd.id
  FROM device_t d,
       dataitem_t di,
       devicedataitem_t ddi,
       collecteddata_t cd
  WHERE ddi.device = d.id AND
        ddi.dataitem = di.id AND
        cd.devicedataitem = ddi.id AND
        di.description = 'totalPages' AND
        cd.ts BETWEEN %s AND %s
  ORDER BY cd.id
        
"""
  
  cur.execute(query, (datetime.datetime(1970, 1, 1), datetime.datetime(2100, 12, 31)))
  
  printers = {}
  
  for (printerName, printerAddress, timestamp, totalPages, id) in cur:
    totalPages = int(totalPages)
    if not printers.has_key(printerAddress):
      printers[printerAddress] = {'name': printerName, 'address': printerAddress, 
                                  'first': {'timestamp': timestamp, 'value': totalPages},
                                  'last': {'timestamp': timestamp, 'value': totalPages}}
    else:
      printers[printerAddress]['last'] = {'timestamp': timestamp, 'value': totalPages}
      

  secondsPerMonth = datetime.timedelta(30).total_seconds()
  secondsPerWeek = datetime.timedelta(7).total_seconds()
  
  xls = xlwt.Workbook()
  mySheet = MySheet(xls.add_sheet('Sheet 1'))
  mySheet.writeFirstFirst('Name')
  mySheet.writeSameNext('Address')
  mySheet.writeSameNext('First Sample Date')
  mySheet.writeSameNext('First Sample Value')
  mySheet.writeSameNext('Last Sample Date')
  mySheet.writeSameNext('Last Sample Value') 
  mySheet.writeSameNext('Diff in SampleBox')
  mySheet.writeSameNext('MonthFactor')
  mySheet.writeSameNext('WeekFactor')
  mySheet.writeSameNext('Month Average/Forecast')
  mySheet.writeSameNext('Week Average/Forecast')
  
  
  for printer in printers.values():
    
    print("Printer: %s, %s" % (printer['name'], printer['address']))  
    mySheet.writeNextFirst(printer['name'])
    mySheet.writeSameNext(printer['address'])
    
    print("  First: %s, %s" % (printer['first']['timestamp'], printer['first']['value']))
    mySheet.writeSameNext(printer['first']['timestamp'])
    mySheet.writeSameNext(printer['first']['value'])
    
    print("  Last:  %s, %s" % (printer['last']['timestamp'], printer['last']['value']))
    mySheet.writeSameNext(printer['last']['timestamp'])
    mySheet.writeSameNext(printer['last']['value'])

    deltaTime = printer['last']['timestamp'] - printer['first']['timestamp']
    deltaValue = printer['last']['value'] - printer['first']['value']
    print("  Diff:  %s, %s" % (deltaTime, deltaValue))
    mySheet.writeSameNext(deltaValue)

    factorMonth = secondsPerMonth / deltaTime.total_seconds()
    mySheet.writeSameNext(factorMonth)
    factorWeek = secondsPerWeek / deltaTime.total_seconds()
    mySheet.writeSameNext(factorWeek)

    print("  Month average/forecast: %s" % (int(deltaValue * factorMonth)))
    mySheet.writeSameNext(int(deltaValue * factorMonth))
    
    print("  Week average/forecast:  %s" % (int(deltaValue * factorWeek)))
    mySheet.writeSameNext(int(deltaValue * factorWeek))
  
  
except mysql.connector.Error as err:
  print("Failure when using database: %s" % (err))
finally:
  if xls is not None:
    xls.save('printerStats.xls')
  if cur is not None:
    cur.close()
  if dbh is not None:
    dbh.close()

